@@ -1,15 +1,21 @@
 __all__ = ['Distribution']
 
 import re
+import os
+import sys
+import warnings
+import numbers
+import distutils.log
+import distutils.core
+import distutils.cmd
+import distutils.dist
 from distutils.core import Distribution as _Distribution
+from distutils.errors import (DistutilsOptionError, DistutilsPlatformError,
+    DistutilsSetupError)
+
 from setuptools.depends import Require
-from setuptools.command.install import install
-from setuptools.command.sdist import sdist
-from setuptools.command.install_lib import install_lib
-from distutils.errors import DistutilsOptionError, DistutilsPlatformError
-from distutils.errors import DistutilsSetupError
-import setuptools, pkg_resources, distutils.core, distutils.dist, distutils.cmd
-import os, distutils.log
+from setuptools.compat import basestring, PY2
+import pkg_resources
 
 def _get_unpatched(cls):
     """Protect against re-patching the distutils if reloaded
@@ -27,6 +33,27 @@ def _get_unpatched(cls):
 
 _Distribution = _get_unpatched(_Distribution)
 
+def _patch_distribution_metadata_write_pkg_info():
+    """
+    Workaround issue #197 - Python 3 prior to 3.2.2 uses an environment-local
+    encoding to save the pkg_info. Monkey-patch its write_pkg_info method to
+    correct this undesirable behavior.
+    """
+    environment_local = (3,) <= sys.version_info[:3] < (3, 2, 2)
+    if not environment_local:
+        return
+
+    # from Python 3.4
+    def write_pkg_info(self, base_dir):
+        """Write the PKG-INFO file into the release tree.
+        """
+        with open(os.path.join(base_dir, 'PKG-INFO'), 'w',
+                  encoding='UTF-8') as pkg_info:
+            self.write_pkg_file(pkg_info)
+
+    distutils.dist.DistributionMetadata.write_pkg_info = write_pkg_info
+_patch_distribution_metadata_write_pkg_info()
+
 sequence = tuple, list
 
 def check_importable(dist, attr, value):
@@ -100,7 +127,8 @@ def check_entry_points(dist, attr, value):
     """Verify that entry_points map is parseable"""
     try:
         pkg_resources.EntryPoint.parse_map(value)
-    except ValueError, e:
+    except ValueError:
+        e = sys.exc_info()[1]
         raise DistutilsSetupError(e)
 
 def check_test_suite(dist, attr, value):
@@ -129,38 +157,6 @@ def check_packages(dist, attr, value):
                 "WARNING: %r not a valid package name; please use only"
                 ".-separated package names in setup.py", pkgname
             )
-            
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 
 
 class Distribution(_Distribution):
@@ -191,7 +187,8 @@ class Distribution(_Distribution):
         EasyInstall and requests one of your extras, the corresponding
         additional requirements will be installed if needed.
 
-     'features' -- a dictionary mapping option names to 'setuptools.Feature'
+     'features' **deprecated** -- a dictionary mapping option names to
+        'setuptools.Feature'
         objects.  Features are a portion of the distribution that can be
         included or excluded based on user options, inter-feature dependencies,
         and availability on the current system.  Excluded features are omitted
@@ -245,10 +242,13 @@ class Distribution(_Distribution):
             dist._version = pkg_resources.safe_version(str(attrs['version']))
             self._patched_dist = dist
 
-    def __init__ (self, attrs=None):
+    def __init__(self, attrs=None):
         have_package_data = hasattr(self, "package_data")
         if not have_package_data:
             self.package_data = {}
+        _attrs_dict = attrs or {}
+        if 'features' in _attrs_dict or 'require_features' in _attrs_dict:
+            Feature.warn_deprecated()
         self.require_features = []
         self.features = {}
         self.dist_files = []
@@ -264,7 +264,7 @@ class Distribution(_Distribution):
             if not hasattr(self,ep.name):
                 setattr(self,ep.name,None)
         _Distribution.__init__(self,attrs)
-        if isinstance(self.metadata.version, (int,long,float)):
+        if isinstance(self.metadata.version, numbers.Number):
             # Some people apparently take "version number" too literally :)
             self.metadata.version = str(self.metadata.version)
 
@@ -281,11 +281,13 @@ class Distribution(_Distribution):
 
     def fetch_build_eggs(self, requires):
         """Resolve pre-setup requirements"""
-        from pkg_resources import working_set, parse_requirements
-        for dist in working_set.resolve(
-            parse_requirements(requires), installer=self.fetch_build_egg
-        ):
-            working_set.add(dist)
+        resolved_dists = pkg_resources.working_set.resolve(
+            pkg_resources.parse_requirements(requires),
+            installer=self.fetch_build_egg,
+            replace_conflicting=True,
+        )
+        for dist in resolved_dists:
+            pkg_resources.working_set.add(dist, replace=True)
 
     def finalize_options(self):
         _Distribution.finalize_options(self)
@@ -318,7 +320,7 @@ class Distribution(_Distribution):
                 'find_links', 'site_dirs', 'index_url', 'optimize',
                 'site_dirs', 'allow_hosts'
             )
-            for key in opts.keys():
+            for key in list(opts):
                 if key not in keep:
                     del opts[key]   # don't use any other settings
             if self.dependency_links:
@@ -359,23 +361,6 @@ class Distribution(_Distribution):
         self.global_options = self.feature_options = go + self.global_options
         self.negative_opt = self.feature_negopt = no
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
     def _finalize_features(self):
         """Add/remove features and resolve dependencies between them"""
 
@@ -393,7 +378,6 @@ class Distribution(_Distribution):
                 feature.exclude_from(self)
                 self._set_feature(name,0)
 
-
     def get_command_class(self, command):
         """Pluggable version of get_command_class()"""
         if command in self.cmdclass:
@@ -413,10 +397,6 @@ class Distribution(_Distribution):
                 self.cmdclass[ep.name] = cmdclass
         return _Distribution.print_commands(self)
 
-
-
-
-
     def _set_feature(self,name,status):
         """Set feature's inclusion status"""
         setattr(self,self._feature_attrname(name),status)
@@ -431,8 +411,8 @@ class Distribution(_Distribution):
         if self.feature_is_included(name)==0:
             descr = self.features[name].description
             raise DistutilsOptionError(
-               descr + " is required, but was excluded or is not available"
-           )
+                descr + " is required, but was excluded or is not available"
+            )
         self.features[name].include_in(self)
         self._set_feature(name,1)
 
@@ -480,7 +460,6 @@ class Distribution(_Distribution):
                     if p.name != package and not p.name.startswith(pfx)
             ]
 
-
     def has_contents_for(self,package):
         """Return true if 'exclude_package(package)' would do something"""
 
@@ -490,15 +469,6 @@ class Distribution(_Distribution):
             if p==package or p.startswith(pfx):
                 return True
 
-
-
-
-
-
-
-
-
-
     def _exclude_misc(self,name,value):
         """Handle 'exclude()' for list/tuple attrs without a special handler"""
         if not isinstance(value,sequence):
@@ -568,18 +538,7 @@ class Distribution(_Distribution):
             raise DistutilsSetupError(
                 "packages: setting must be a list or tuple (%r)" % (packages,)
             )
-        map(self.exclude_package, packages)
-
-
-
-
-
-
-
-
-
-
-
+        list(map(self.exclude_package, packages))
 
     def _parse_command_opts(self, parser, args):
         # Remove --with-X/--without-X options when processing command args
@@ -607,21 +566,6 @@ class Distribution(_Distribution):
 
         return nargs
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
     def get_cmdline_options(self):
         """Return a '{cmd: {opt:val}}' map of all command-line options
 
@@ -662,7 +606,6 @@ class Distribution(_Distribution):
 
         return d
 
-
     def iter_distribution_names(self):
         """Yield all packages, modules, and extension names in distribution"""
 
@@ -681,7 +624,6 @@ class Distribution(_Distribution):
                 name = name[:-6]
             yield name
 
-
     def handle_display_options(self, option_order):
         """If there were any non-global "display-only" options
         (--help-commands or the metadata display options) on the command
@@ -690,7 +632,7 @@ class Distribution(_Distribution):
         """
         import sys
 
-        if sys.version_info < (3,) or self.help_commands:
+        if PY2 or self.help_commands:
             return _Distribution.handle_display_options(self, option_order)
 
         # Stdout may be StringIO (e.g. in tests)
@@ -723,26 +665,14 @@ for module in distutils.dist, distutils.core, distutils.cmd:
     module.Distribution = Distribution
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 class Feature:
-    """A subset of the distribution that can be excluded if unneeded/wanted
+    """
+    **deprecated** -- The `Feature` facility was never completely implemented
+    or supported, `has reported issues
+    <https://bitbucket.org/pypa/setuptools/issue/58>`_ and will be removed in
+    a future version.
+
+    A subset of the distribution that can be excluded if unneeded/wanted
 
     Features are created using these keyword arguments:
 
@@ -791,9 +721,19 @@ class Feature:
     Aside from the methods, the only feature attributes that distributions look
     at are 'description' and 'optional'.
     """
+
+    @staticmethod
+    def warn_deprecated():
+        warnings.warn(
+            "Features are deprecated and will be removed in a future "
+                "version. See http://bitbucket.org/pypa/setuptools/65.",
+            DeprecationWarning,
+            stacklevel=3,
+        )
+
     def __init__(self, description, standard=False, available=True,
-        optional=True, require_features=(), remove=(), **extras
-    ):
+            optional=True, require_features=(), remove=(), **extras):
+        self.warn_deprecated()
 
         self.description = description
         self.standard = standard
@@ -844,8 +784,6 @@ class Feature:
         for f in self.require_features:
             dist.include_feature(f)
 
-
-
     def exclude_from(self,dist):
 
         """Ensure feature is excluded from distribution
@@ -862,8 +800,6 @@ class Feature:
             for item in self.remove:
                 dist.exclude_package(item)
 
-
-
     def validate(self,dist):
 
         """Verify that feature makes sense in context of distribution
@@ -883,7 +819,3 @@ class Feature:
                     " doesn't contain any packages or modules under %s"
                     % (self.description, item, item)
                 )
-
-
-
-
